from openpyxl import Workbook
import re
import os
from .table_merge import Table_Merge
from .table_attr import Table_Attr


class Header:
    def __init__(self):
        self.headers = []
        self.aliases = {}
        self.header_col = 0
        self.record_keys = None
        self.sort_keys = None
        self.formulas = {}
        self.hash_key = None

    def add(self, header_list):       
        if isinstance(header_list, list) and all(isinstance(h, str) for h in header_list):
            if self.header_col == 0:            
                self.headers.append(header_list)
                self.header_col = len(header_list)
                self.record_keys = header_list
            else:                
                if self.header_col != len(header_list):
                    raise ValueError("多个Header参数长度不匹配")                    
                self.headers.append(header_list)
            
        else:   
            raise ValueError("Header 参数必须是字符串列表")

    def set_active(self, index, sort_keys):
        if isinstance(index, int) and 0 <= index < len(self.headers):
            self.record_keys = self.headers[index].copy()
            self.sort_keys = sort_keys
        else:
            raise ValueError("索引必须是有效的整数")
        
        
    def set_head_formula(self, head_formula):
        def __extract_patterns(input_string):
            # 使用正则表达式匹配 ${...} 格式的字符串，并提取所有匹配项
            patterns = re.findall(r'\$\{(.+?)\}', input_string)            
            return patterns
        
        def __index_to_column(index):
            if index < 0:
                raise ValueError("Index must be a non-negative integer")            
            # 将索引转换为 ASCII 值，其中 65 对应于 'A'
            column_char = chr(index + 65)
            return column_char
        
        def __replace_string(input_string, old_substring, new_substring):
            # 删除所有空白字符
            ss = ''.join(input_string.split())

            # 使用 str.replace 方法替换所有匹配的子字符串
            return ss.replace(old_substring, new_substring)

    
        # 检查 head_formula_list 是否为字典
        if not isinstance(head_formula, dict):
            raise TypeError("head_formula_list must be a dictionary")

        # 检查字典的键和值是否都是字符串
        for key, value in head_formula.items():
            if not isinstance(key, str) or not isinstance(value, str):
                raise TypeError("Both keys and values in the dictionary must be strings")
            
        if not self.record_keys:
            raise TypeError("set_head_formula() depends on set_active().")
       
        for formula_result_key, formula_str in head_formula.items():
            formula_inputs_key = __extract_patterns(formula_str)
            self.formulas[formula_result_key] = { 'inputs' : [], 'excel_formula' : None }
            forumla = self.formulas[formula_result_key]
            
            if formula_result_key not in self.record_keys:
                raise ValueError("formula_result_key must be in record_keys")
            
            for input in formula_inputs_key:                
                input_formula = {'index' : None, 'key' : None, 'excel_pos' : None}
                if input not in self.record_keys:
                    raise ValueError("formula_inputs_key must be in record_keys")
                input_formula['index'] = self.record_keys.index(input)
                input_formula['key'] = input
                input_formula['excel_pos'] = __index_to_column(input_formula['index'])
                forumla['inputs'].append(input_formula)
                formula_str = __replace_string(formula_str, "${" + input + "}", input_formula['excel_pos']+"#N")
            
            forumla['excel_formula'] = "="  + formula_str
                

    def set_alias(self, alias_dict):
        if isinstance(alias_dict, dict) and all(isinstance(k, str) and isinstance(v, str) for k, v in alias_dict.items()):
            self.aliases.update(alias_dict)
        else:
            raise ValueError("字典的键和值必须是字符串")
        
    def set_hash_key(self, hash_key):
        if not isinstance(hash_key, list):
            raise ValueError("hash_key 必须是列表") 
        
        for key in hash_key:
            if key not in self.record_keys:
                raise ValueError("hash_key 必须是record_keys的子集")
        self.hash_key = hash_key
        
        
    def __str__(self):
        header_str = "Headers:\n" + "\n".join(", ".join(header) for header in self.headers)
        alias_str = "Aliases:\n" + "\n".join(f"{k}: {v}" for k, v in self.aliases.items())
        active_header_str = "Active Header:\n" + ", ".join(self.record_keys) if self.record_keys else "None"
        return f"{header_str}\n\n{alias_str}\n\n{active_header_str}"
        
    


class Record:
    def __init__(self):
        self.records = []
        
    # 解析字符串获取类似(key@value)的字典
    @staticmethod
    def __parse_line(line, tag_name):
        if tag_name not in line:
            return {}
        pattern = r'\(([^@]+)@([^)]+)\)'
        matches = re.findall(pattern, line)
        
        result = {}
        for key, value in matches:
            # 尝试将 value 转换为整数
            try:
                value = int(value)
            except ValueError:
                # 如果无法转换为整数，则尝试转换为浮点数
                try:
                    value = float(value)
                except ValueError:
                    # 如果无法转换为浮点数，则保持为字符串
                    pass
            
            result[key] = value

        return result


    def add_from_str(self, tag_name, record_str):
        if isinstance(record_str, str):
            lines = record_str.split('\n')
            for line in lines:
                one = Record.__parse_line(line, tag_name)
                if one:
                    self.records.append(one)
        else:
            raise ValueError("参数必须是字符串")
        
    def add_from_file(self, tag_name, log_file):
        # 检查log_file是否为可读文件
        if not os.path.isfile(log_file) or not os.access(log_file, os.R_OK):
            raise ValueError(f"log_file must be a readable file. Got: {log_file}")

        # 打开文件并逐行读取
        with open(log_file, 'r', encoding='utf-8') as file:
            for line in file:
                one = Record.__parse_line(line, tag_name)
                if one:
                    self.records.append(one)

    def add_from_dict(self, record_dict):
        if isinstance(record_dict, dict):
            self.records.append(record_dict)
        else:
            raise ValueError("参数必须是字典")
        
    def __str__(self):
        records_str = "\n".join(str(record) for record in self.records)
        return f"Records:\n{records_str}"


class Table:
    def __init__(self, name_tag, worksheet, header, records):
        if not isinstance(header, Header) or not isinstance(records, list):
            raise TypeError("header and record must be Header and Record objects respectively.")
        self.__worksheet = worksheet
        self.__header = header
        self.__records = records       
        self.__table_info = {'first_table': True, 'col_width': 1, 'header' : {'row_start':None, 'row_end':None}, 'record' : {'row_start':None, 'row_end':None}}        
        self.__sort_key_row_index = None
        self.__name_tag = name_tag
        
        
        # replace the header name with alias name
        for h in self.__header.headers:
            for i, l in enumerate(h):
                if l in header.aliases:
                    h[i] = header.aliases[l]
                    
        # check self.header.record_keys list strings cannot be the same
        if len(set(self.__header.record_keys)) != len(self.__header.record_keys):
            raise ValueError("用户设置header的record_keys 包含重复字符串")
        
        self.__to_excel()

            
    def __expand_and_sort_by_keys(self, records, record_keys, sort_keys):
        # 检查sort_keys是否是record_keys的子集
        if not all(key in record_keys for key in sort_keys):
            raise ValueError("sort_keys必须是record_keys的子集")
        
        # expand
        for index, record in enumerate(records):
            if isinstance(record, dict):
                # 检查record的所有键是否都在record_keys中
                if not all(key in record_keys for key in record.keys()):
                    raise ValueError("所有的record的keys都必须在record_keys中")
                
                # 检查record的必须有所有的sort_keys
                if any(not key in record.keys() for key in sort_keys):
                    raise ValueError("record的少关键key, record = {}, 关键keys = {}".format(record, sort_keys))
                
                records[index] = [record.get(key, None) for key in record_keys]
            else:
                raise TypeError("记录必须是字典类型")

        # sort 
        # 计算sort_keys在record_keys的索引列表
        sort_indexes = [record_keys.index(key) for key in sort_keys]

        # 根据sort_indexes对self.records进行排序
        records.sort(key=lambda record: [record[index] if isinstance(record[index], (int, float)) else str(record[index]) for index in sort_indexes])
        
        self.__sort_key_row_index = sort_indexes
    
    
    def merge_cells(self):
        table_merge = Table_Merge(self.__worksheet, self.__table_info, self.__sort_key_row_index)
        table_merge.merge()        
        
    def set_attrs(self):
        table_attr = Table_Attr(self.__worksheet, self.__table_info, self.__sort_key_row_index)
        table_attr.set_attr()
        pass
        
    def __str__(self):
        # 获取表格信息中的起始和结束行
        row_start = self.__table_info['header']['row_start']
        row_end = self.__table_info['record']['row_end']

        # 使用列表推导式构建一个包含指定行范围的字符串列表
        rows = ['\t'.join(map(str, row)) for row in self.__worksheet.iter_rows(min_row=row_start, max_row=row_end, values_only=True)]

        # 使用换行符连接所有行，构建一个完整的字符串
        return '\n'.join(rows)    
    
    @staticmethod
    def __expand_record_by_formula(record, row_num, formulas, record_keys):
        for formula_result_key, formula in formulas.items():
            output_ind = record_keys.index(formula_result_key)            
            
            inputs_valid = True
            for input in formula['inputs']:
                input_ind = input['index']
                # 输入为None，那么不用公式计算
                if record[input_ind] is None:
                    inputs_valid = False
                    break
                
                 # 支持公式嵌套，允许cell A公式的输出做为cell B的公式输入                
                if isinstance(record[input_ind], (int, float)) is False:
                    if isinstance(record[input_ind], (str)) is True and not record[input_ind].startswith("="):
                        inputs_valid = False
                    break
            
            # 只有输入有数值、或者为公式，且输出为空，那么才用公式计算
            if inputs_valid and record[output_ind] is None:
                record[output_ind] = formula['excel_formula'].replace("#N", str(row_num))
            pass
        
        return record
    
    @staticmethod
    def __combine_two_records(record1, record2):
        for k, v in record2.items():
            if v is not None and k not in record1:
                record1[k] = v
        return record1
        pass
       
    def __to_excel(self):        
        # 计算表格信息
        worksheet = self.__worksheet
        if worksheet.max_row == 1:
            self.__table_info['header']['row_start'] = worksheet.max_row
        else:
            # 同一个sheet中有多个table，那么table之间保留间隔行
            self.__table_info['header']['row_start'] = worksheet.max_row + 3
            worksheet.append([])
            worksheet.append([])
            self.__table_info['first_table'] = False
            
        self.__table_info['header']['row_end'] = self.__table_info['header']['row_start'] + len(self.__header.headers) - 1
        self.__table_info['record']['row_start'] = self.__table_info['header']['row_end'] + 1
        self.__table_info['record']['row_end'] = self.__table_info['record']['row_start'] + len(self.__records[0].records) - 1
       
        # 将表头写入工作表        
        for header in self.__header.headers:
            worksheet.append(header) 
            self.__table_info['col_width'] = max( self.__table_info['col_width'], len(header) )
               
               
        if len(self.__records) > 1 and self.__header.hash_key is None:
            raise ValueError("多个record必须设置hash_key")
        
        # 把多个record对象的数据合并到第一个record对象中
        first_record_obj = self.__records[0]
        if self.__header.hash_key:
            others_record_obj = self.__records[1:]
            for record in first_record_obj.records:
                hash_val = hash(tuple(record[key] for key in self.__header.hash_key))
                for other_record_obj in others_record_obj:
                    for other_record in other_record_obj.records:
                        other_hash_val = hash(tuple(other_record[key] for key in self.__header.hash_key))
                        if hash_val == other_hash_val:
                            record = Table.__combine_two_records(record, other_record)
                            break
                pass
        
        
        
        # 以第一个记录表排序
        self.__expand_and_sort_by_keys(self.__records[0].records, self.__header.record_keys, self.__header.sort_keys)
        record_row_start = worksheet.max_row + 1

        for record in self.__records[0].records:                
            record = self.__expand_record_by_formula(record, worksheet.max_row + 1, self.__header.formulas, self.__header.record_keys)
            worksheet.append(record)
        record_row_end = worksheet.max_row



from openpyxl import Workbook
from openpyxl.worksheet.worksheet import Worksheet
from .table import Header, Record, Table
import os
import yaml


TEMPLATE_CFG_FILE = os.path.join(os.path.dirname(__file__), '../data/template.yaml')

class Sheet:
    def __init__(self, worksheet, name='Sheet1'):
        if not isinstance(worksheet, Worksheet):
            raise TypeError("worksheet must be an instance of openpyxl.worksheet.worksheet.Worksheet")

        self.__sheet = worksheet
        self.__name = name


    def __str__(self):
        # 添加工作表名称
        sheet_name_info = f"------- Sheet Name: {self.__name}"

        # 使用列表推导式构建一个包含所有行的字符串列表
        rows = ['\t'.join(map(str, row)) for row in self.__sheet.iter_rows(values_only=True)]

        # 添加合并单元格的信息
        merged_cells_info = "Merged Cells:"
        for merged_range in self.__sheet.merged_cells.ranges:
            merged_cells_info += f"\n{merged_range.coord}"

        # 使用换行符连接工作表名称、所有行和合并单元格的信息，构建一个完整的字符串
        return sheet_name_info + '\n' + '\n'.join(rows) + '\n' + merged_cells_info



def get_default_config(output_file_path):
    # 检查并创建输出目录（如果不存在）
    output_directory = os.path.dirname(output_file_path)
    if output_directory and not os.path.exists(output_directory):
        os.makedirs(output_directory)
        
    with open(TEMPLATE_CFG_FILE, 'r', encoding='utf-8') as template_file:        
        with open(output_file_path, 'w', encoding='utf-8') as output_file:            
            for line in template_file:
                output_file.write(line)
                


def gen_excel_table(config_file):
    config = {}
    excel_path = None
    sheet_tags = None
    # 检查文件是否存在
    if not os.path.isfile(config_file):
        raise ValueError(f"File {config_file} does not exist.")

    # 尝试加载YAML内容以检查其有效性
    try:
        with open(config_file, 'r', encoding='utf-8') as file:
            config = yaml.safe_load(file)
    except yaml.YAMLError:
        raise ValueError(f"File {config_file} is not a valid YAML file.")
    
    
    # excel_path处理
    excel_path = config.get('excel_path')
    if not excel_path or not excel_path.endswith('.xlsx'):
        raise ValueError(f"excel_path must be a .xlsx file. Got: {excel_path}")
    if os.path.exists(excel_path) and not os.access(excel_path, os.W_OK):
        raise ValueError(f"excel_path is not writable. Got: {excel_path}")
    
    # 创建一个工作簿和工作表
    workbook = Workbook()
    
    # 删除缺省sheet
    workbook.remove(workbook.active)

    
    # sheet处理
    sheets = config.get('sheets')
    if not isinstance(sheets, dict) or not isinstance(sheets.get('tag'), list) or not all(isinstance(item, str) for item in sheets.get('tag', [])):
        raise ValueError("sheets must be a dictionary containing a 'tag' key with a list of strings.")
    
    
    excel_sheets = {}
    for sheet_tag in sheets['tag']:        
        excel_sheets[sheet_tag] = workbook.create_sheet(title=sheet_tag)            
        
    
    
    # table处理    
    tables = config.get('tables', [])
    for table in tables:
        if 'name' not in table:
            raise ValueError(f"{config_file} failed, no table name")
        
        # table 的sheet tag处理
        if 'sheet_tag' not in table:
            raise ValueError(f"{config_file} failed,  no table sheet tag")
        
        if table['sheet_tag'] not in sheets['tag']:
            raise ValueError(f"{config_file} failed,  table sheet is not a valid sheet name, sheets = {table['sheet_tag']}")        
        table_sheet = excel_sheets[table['sheet_tag']]
        
        #header 处理
        header = Header()
        if 'head-key' not in table:
            raise ValueError(f"{config_file} failed,  no table head-key")
        
        if 'head-0' in table:
            header.add(table['head-0'])
        else:
            raise ValueError(f"{config_file} failed,  no table head-0")
        
        if 'head-1' in table:
            header.add(table['head-1'])
            header.set_active(1, table['head-key'])
        else:
            header.set_active(0, table['head-key'])
            
        if 'head-formula' in table:
            header.set_head_formula(table['head-formula'])
            
        if 'alias' in table:
            header.set_alias(table['alias'])
            
            
        if 'head-hash-key' in table:
            header.set_hash_key(table['head-hash-key'])
                
        # record 处理        
        record_files = table.get('record_file')        
        if not isinstance(record_files, list):
            record_files = [record_files]            
        
        record_list = []
        for record_file in record_files:
            if not record_file or not os.path.isfile(record_file) or not os.access(record_file, os.R_OK):     
                table['record_file'] = os.path.join(os.path.dirname(__file__), table['record_file'])
                record_file = table['record_file']
                if not os.path.isfile(record_file) or not os.access(record_file, os.R_OK):
                    raise ValueError(f"record_file must be a readable file. Got: {record_file}")

            record = Record() 
            record.add_from_file(table['name'], record_file)
            if not record.records:
                raise ValueError(f"could not extract table record from  {record_file}, table_tag is {table['name']} ")
            
            record_list.append(record)
        
        # table处理
        table = Table(table['name'], table_sheet, header, record_list)
        table.merge_cells()
        table.set_attrs()
        pass
        
        
        
        # print(Sheet(table_sheet))
        
    
    
    # 保存工作簿
    workbook.save(excel_path)
    print(f"Excel file {excel_path} is generated by {config_file}")


import unittest
from ptable import gen_excel_table, get_default_config

import os
TEST_CONFIG_file = os.path.join(os.path.dirname(__file__), '../data/test_cfg.yaml')
DEMO_CONFIG_file = os.path.join(os.path.dirname(__file__), '../data/demo_cfg.yaml')
DEMO_LOG_FILE = os.path.join(os.path.dirname(__file__), '../data/demo_log.txt')

class TestSheet(unittest.TestCase):
    
    def test_sheet_case01(self):
        if os.path.isfile(TEST_CONFIG_file):
            os.remove(TEST_CONFIG_file)
        get_default_config(TEST_CONFIG_file) 
        self.assertEqual(os.path.isfile(TEST_CONFIG_file), True)
        pass
    
    def test_sheet_case02(self):
        gen_excel_table(DEMO_CONFIG_file)
        pass
 

# 如果是直接运行这个文件，那么执行测试
if __name__ == '__main__':
    unittest.main()



import unittest
from openpyxl import Workbook

from ptable import Header, Record, Table, Sheet
import os
OUTPUT_DIR = os.path.join(os.path.dirname(__file__), '../data/')


class TestTable(unittest.TestCase):
    
    def test_table_case01(self):
        header = Header()
        header.add(["model", "thread", "step","device_nn", "device_nn", "host_pcie_nn", "host_pcie_nn"])
        header.add(["model", "thread", "step","dev_ms",    "dev_fps",   "host_ms",      "host_fps"])
        header.set_active(1, ['model','thread'])
        header.set_alias({"model": "模型", 
                          "thread": "线程数", 
                          "step": "步骤", 
                          "dev_ms": "时间(ms)", 
                          "dev_fps": "帧率", 
                          "host_ms": "时间(ms)", 
                          "host_fps": "帧率",
                          "device_nn":"device推理",
                          "host_pcie_nn":"Host PCIe推理"})
        record = Record()
        
        record.add_from_dict({"model": "resnet50", 
                              "thread": '2', 
                              "step" :  "dclmdlLoadFromFile",
                              "dev_ms" : "2",
                              "host_ms" : "1.59"})

        record.add_from_dict({"model": "resnet50", 
                              "thread": '1', 
                              "step" :  "dclmdlLoadFromFile",
                              "dev_ms" : "2.59",
                              "host_ms" : "3.59"})

        record.add_from_dict({"model": "resnet50", 
                              "thread": '1', 
                              "step" :  "copyH2D",
                              "host_ms" : "0.5"})

        record.add_from_dict({"model": "resnet50", 
                              "thread": '1', 
                              "step" :  "copyD2H",
                              "host_ms" : "1.5"})

        record.add_from_dict({"model": "yolov8", 
                              "thread": '16', 
                              "step" :  "copyD2H",
                              "host_ms" : "3.5"})

        record.add_from_dict({"model": "resnet50", 
                              "thread": '2', 
                              "step" :  "copyH2D",
                              "host_ms" : "0.5"})

        record.add_from_dict({"model": "resnet50", 
                              "thread": '4', 
                              "step" :  "copyD2H",
                              "host_ms" : "3.5"})

        print(record)

        # 创建一个工作簿和工作表
        workbook = Workbook()
        worksheet = workbook.active

        table = Table('#tab1', worksheet, header, [record])
        table.merge_cells()
        table.set_attrs()
        
        
                
        print(table)        
        print(Sheet(worksheet,"nnp perf"))

         # 保存工作簿
        workbook.save(OUTPUT_DIR + "nnp-perf-case01.xlsx")
        
        
    def test_table_case02(self):
        header = Header()
        header.add(["$1", "input",      "input",    "output",       "output",   "$d1",      "$d1",        "$d1_copy",     "$d1_copy",       "h2d_size", "h2d_time", "ave_bw"])
        header.add(["$1", "in.format", "in.res",   "out.format",  "out.res",  "$d1.fps",  "$d1.rate",   "$d1_copy.fps", "$d1_copy.rate",  "h2d_size", "h2d_time", "ave_bw"])
        header.set_active(1, ['$1','in.format','out.format'])
        header.set_alias({"$1": "处理类别", 
                          "input": "输入", 
                          "in.format": "格式", 
                          "in.res": "分辨率", 
                          "output": "输出", 
                          "out.format": "格式", 
                          "out.res": "分辨率", 
                          
                          "$d1": "单device", 
                          "$d1.fps": "FPS", 
                          "$d1.rate": "最大rate", 
                          
                          "$d1_copy": "单device(拷贝)", 
                          "$d1_copy.fps": "FPS", 
                          "$d1_copy.rate": "最大rate", 
                          
                          
                          "h2d_size": "h2d传输大小(MB)", 
                          "h2d_time": "h2d传输时间(ms)", 
                          "ave_bw": "平均带宽(MB/S)"})
        
        
        record = Record()        
        record.add_from_dict({"$1": "Resize", 
                              "in.format": 'YUV400', 
                              "in.res": '1920x1080', 
                              "out.format": 'YUV400', 
                              "out.res": '224x224',
                              "$d1.fps" : '8109.63',
                              "$d1.rate" : '100',   
                              "$d1_copy.fps" : '2572.07', 
                              "$d1_copy.rate" : '33',  
                              "h2d_size" : '2', 
                              "h2d_time" : '2.887', 
                              "ave_bw" : '692.76'})
        record.add_from_str("#tab1", """---demo : add record by logs
                            #tab1: ($1@Resize),(in.format@YUV400),(in.res@640x480),(out.format@YUV400),(out.res@224x224),($d1.fps@18969.5),($d1.rate@100),($d1_copy.fps@18468),($d1_copy.rate@99),(h2d_size@0.29),(h2d_time@0.63)
                            ....
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@1920x1080),(out.format@YUV400),(out.res@224x224),  ($d1.fps@2690.19),($d1.rate@100),($d1_copy.fps@853.268),($d1_copy.rate@35),(h2d_size@5.93),(h2d_time@7.715)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@256x256),(out.format@YUV400),  (out.res@1024x1024),($d1.fps@947.92), ($d1.rate@100),($d1_copy.fps@946.962),($d1_copy.rate@100),(h2d_size@0.1875),(h2d_time@0.484)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@1920x1080),(out.format@YUV400),(out.res@1024x1024),($d1.fps@545.703),($d1.rate@100),($d1_copy.fps@503.539),($d1_copy.rate@98),(h2d_size@5.93),(h2d_time@7.683)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@1920x1080),(out.format@YUV400),(out.res@2048x2048),($d1.fps@237.727),($d1.rate@100),($d1_copy.fps@226.873),($d1_copy.rate@100),(h2d_size@5.93),(h2d_time@7.683)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@3840x2160),(out.format@YUV400),(out.res@1024x1024),($d1.fps@290.252),($d1.rate@100),($d1_copy.fps@189.045),($d1_copy.rate@72),(h2d_size@23.95),(h2d_time@29.821)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@3840x2160),(out.format@YUV400),(out.res@2048x2048),($d1.fps@139.121),($d1.rate@100),($d1_copy.fps@128.14), ($d1_copy.rate@98),(h2d_size@23.95),(h2d_time@29.821)
                            ...
                            #tab1: ($1@CvtColor),(in.format@YUV420),(in.res@1920x1080),(out.format@RGB_PLANAR/RGB_PACKED/YUV444),(out.res@1920x1080),($d1.fps@480.366),($d1.rate@100),($d1_copy.fps@480.282), ($d1_copy.rate@100),(h2d_size@3),(h2d_time@4.097)
                            #tab1: ($1@CvtColor),(in.format@YUV420),(in.res@1280x720), (out.format@RGB_PLANAR/RGB_PACKED/YUV444),(out.res@1280x720),($d1.fps@1082.6),($d1.rate@100),($d1_copy.fps@1080.55), ($d1_copy.rate@100),(h2d_size@1.32),(h2d_time@2.044)
                            #tab1: ($1@CvtColor),(in.format@YUV420),(in.res@640x480),  (out.format@RGB_PLANAR/RGB_PACKED/YUV444),(out.res@640x480),($d1.fps@3237.86),($d1.rate@98),($d1_copy.fps@3231.45), ($d1_copy.rate@100),(h2d_size@0.44),(h2d_time@0.851)
                            """)

         # 创建一个工作簿和工作表
        workbook = Workbook()
        worksheet = workbook.active

        table = Table('#tab1', worksheet, header, [record])
        table.merge_cells()
        table.set_attrs()        
                       
        print(table)        
        print(Sheet(worksheet,"ive perf"))

         # 保存工作簿
        workbook.save(OUTPUT_DIR + "ive-perf-case02.xlsx")

   
    def test_table_case03(self):
        header = Header()
        header.add(["$1", "input",      "input",    "output",       "output",   "$d1",      "$d1",        "$d1_copy",     "$d1_copy",       "h2d_size", "h2d_time", "ave_bw"])
        header.add(["$1", "in.format", "in.res",   "out.format",  "out.res",  "$d1.fps",  "$d1.rate",   "$d1_copy.fps", "$d1_copy.rate",  "h2d_size", "h2d_time", "ave_bw"])
        header.set_active(1, ['$1','in.format','out.format'])
        header.set_alias({"$1": "处理类别", 
                          "input": "输入", 
                          "in.format": "格式", 
                          "in.res": "分辨率", 
                          "output": "输出", 
                          "out.format": "格式", 
                          "out.res": "分辨率", 
                          
                          "$d1": "单device", 
                          "$d1.fps": "FPS", 
                          "$d1.rate": "最大rate", 
                          
                          "$d1_copy": "单device(拷贝)", 
                          "$d1_copy.fps": "FPS", 
                          "$d1_copy.rate": "最大rate", 
                          
                          
                          "h2d_size": "h2d传输大小(MB)", 
                          "h2d_time": "h2d传输时间(ms)", 
                          "ave_bw": "平均带宽(MB/S)"})
        
        
        record = Record()        
        record.add_from_dict({"$1": "Resize", 
                              "in.format": 'YUV400', 
                              "in.res": '1920x1080', 
                              "out.format": 'YUV400', 
                              "out.res": '224x224',
                              "$d1.fps" : '8109.63',
                              "$d1.rate" : '100',   
                              "$d1_copy.fps" : '2572.07', 
                              "$d1_copy.rate" : '33',  
                              "h2d_size" : '2', 
                              "h2d_time" : '2.887', 
                              "ave_bw" : '692.76'})
        
        # 乱序排放，验证排序和合并是否准确
        record.add_from_str("#tab1", """---demo : add record by logs
                            #tab1: ($1@Resize),(in.format@YUV400),(in.res@640x480),(out.format@YUV400),(out.res@224x224),($d1.fps@18969.5),($d1.rate@100),($d1_copy.fps@18468),($d1_copy.rate@99),(h2d_size@0.29),(h2d_time@0.63)
                            ....
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@1920x1080),(out.format@YUV400),(out.res@224x224),  ($d1.fps@2690.19),($d1.rate@100),($d1_copy.fps@853.268),($d1_copy.rate@35),(h2d_size@5.93),(h2d_time@7.715)
                            #tab1: ($1@CvtColor),(in.format@YUV420),(in.res@1280x720), (out.format@RGB_PLANAR/RGB_PACKED/YUV444),(out.res@1280x720),($d1.fps@1082.6),($d1.rate@100),($d1_copy.fps@1080.55), ($d1_copy.rate@100),(h2d_size@1.32),(h2d_time@2.044)
                            ...
                            ...
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@256x256),(out.format@YUV400),  (out.res@1024x1024),($d1.fps@947.92), ($d1.rate@100),($d1_copy.fps@946.962),($d1_copy.rate@100),(h2d_size@0.1875),(h2d_time@0.484)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@1920x1080),(out.format@YUV400),(out.res@1024x1024),($d1.fps@545.703),($d1.rate@100),($d1_copy.fps@503.539),($d1_copy.rate@98),(h2d_size@5.93),(h2d_time@7.683)
                            ...
                            #tab1: ($1@CvtColor),(in.format@YUV420),(in.res@1920x1080),(out.format@RGB_PLANAR/RGB_PACKED/YUV444),(out.res@1920x1080),($d1.fps@480.366),($d1.rate@100),($d1_copy.fps@480.282), ($d1_copy.rate@100),(h2d_size@3),(h2d_time@4.097)
                            ...
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@1920x1080),(out.format@YUV400),(out.res@2048x2048),($d1.fps@237.727),($d1.rate@100),($d1_copy.fps@226.873),($d1_copy.rate@100),(h2d_size@5.93),(h2d_time@7.683)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@3840x2160),(out.format@YUV400),(out.res@1024x1024),($d1.fps@290.252),($d1.rate@100),($d1_copy.fps@189.045),($d1_copy.rate@72),(h2d_size@23.95),(h2d_time@29.821)
                            #tab1: ($1@Resize),(in.format@RGB_PLANAR),(in.res@3840x2160),(out.format@YUV400),(out.res@2048x2048),($d1.fps@139.121),($d1.rate@100),($d1_copy.fps@128.14), ($d1_copy.rate@98),(h2d_size@23.95),(h2d_time@29.821)
                            ...
                            #tab1: ($1@CvtColor),(in.format@YUV420),(in.res@640x480),  (out.format@RGB_PLANAR/RGB_PACKED/YUV444),(out.res@640x480),($d1.fps@3237.86),($d1.rate@98),($d1_copy.fps@3231.45), ($d1_copy.rate@100),(h2d_size@0.44),(h2d_time@0.851)
                            """)

         # 创建一个工作簿和工作表
        workbook = Workbook()
        worksheet = workbook.active

        table = Table('#tab1', worksheet, header, [record])
        table.merge_cells()
        table.set_attrs()        
                       
        print(table)        
        print(Sheet(worksheet,"ive perf"))

         # 保存工作簿
        workbook.save(OUTPUT_DIR + "ive-perf-case03.xlsx")
        
    def test_table_case04(self):
        header = Header()
        header.add(["in","size"])      
        header.set_active(0, ['in'])  
        
        record = Record()        
        record.add_from_str("#tab1", """---demo : add record by logs
                            #tab1: (in@1000000),    (size@1024)
                            #tab1: (in@100000),     (size@4096)
                            #tab1: (in@10000),      (size@65536)
                            #tab1: (in@1000),       (size@524288)
                            #tab1: (in@100),        (size@1048576)
                            #tab1: (in@10),         (size@4194304)
                            #tab1: (in@1),          (size@8388608)
                            #tab1: (in@10),         (size@33554432)
                            #tab1: (in@100),        (size@8388608)
                            #tab1: (in@1000),       (size@1048576)
                            #tab1: (in@10000),      (size@65536)                           
                            """)

         # 创建一个工作簿和工作表
        workbook = Workbook()
        worksheet = workbook.active

        table = Table('#tab1', worksheet, header, [record])
        table.merge_cells()
        table.set_attrs()        
                       
        print(table)        
        print(Sheet(worksheet,"number-align"))

         # 保存工作簿
        workbook.save(OUTPUT_DIR + "number-align-case04.xlsx")

# 如果是直接运行这个文件，那么执行测试
if __name__ == '__main__':
    unittest.main()



import unittest
from openpyxl import Workbook
from ptable import Table_Merge, Sheet

class TestTableMerge(unittest.TestCase):
    
    def test_merge_header_case01(self):
        ws = Workbook().active
        sheet = Sheet(ws)
        ws.append(["A"])
        ws.append(["A"])
        table_info = {'header': {'row_start':1, 'row_end' : 2},}
        sort_key_row_index = [0,] # 根据A1排序并合并
        table_merge = Table_Merge(ws, table_info, sort_key_row_index)
        
        table_merge.merge()
        print(sheet)
        
        
        # 使用断言方法验证合并后的单元格
        self.assertEqual(len(ws.merged_cells.ranges), 1)
        for merged_cell in ws.merged_cells.ranges:
            if 'A1' in merged_cell:
                self.assertEqual(merged_cell.coord, "A1:A2")

        
        

    def test_merge_header_case02(self):
        ws = Workbook().active
        sheet = Sheet(ws)
        ws.append(["A",     "A",    "B",    "C",    "C"])
        ws.append(["A1",    "A2",   "B",   "C1",   "C2"])
        table_info = {'header': {'row_start':1, 'row_end' : 2},}
        sort_key_row_index = [0,] # 根据A1排序并合并
        table_merge = Table_Merge(ws, table_info, sort_key_row_index)
        
        table_merge.merge()
        print(sheet)
        
        
        # 使用断言方法验证合并后的单元格
        self.assertEqual(len(ws.merged_cells.ranges), 3)
        for merged_cell in ws.merged_cells.ranges:
            if 'A1' in merged_cell:
                self.assertEqual(merged_cell.coord, "A1:B1")
            if 'D1' in merged_cell:
                self.assertEqual(merged_cell.coord, "D1:E1")
            if 'C1' in merged_cell:
                self.assertEqual(merged_cell.coord, "C1:C2")
        
        
    def test_merge_record_case01(self):
        ws = Workbook().active
        sheet = Sheet(ws)
        ws.append(["A",    "B",   "C",    "D",    "E"])
        ws.append(["A",    "B",   "C",   "D1",   "E1"])
        ws.append(["A",    "B1",   "C",   "D1",   "E2"])
        table_info = {'record': {'row_start':1, 'row_end' : 3},}
        sort_key_row_index = [0, 1] # 根据A,B排序并合并
        table_merge = Table_Merge(ws, table_info, sort_key_row_index)
        table_merge.merge()
        print(sheet)
        
        #仅允许A，B列merge
        self.assertEqual(len(ws.merged_cells.ranges), 2)
        for merged_cell in ws.merged_cells.ranges:
            if 'A1' in merged_cell:
                self.assertEqual(merged_cell.coord, "A1:A3")
            if 'B1' in merged_cell:
                self.assertEqual(merged_cell.coord, "B1:B2")
       
        
        
    def test_merge_record_case02(self):
        ws = Workbook().active
        sheet = Sheet(ws)
        ws.append(["A1",    "B",   "C",    "D",    "E"])
        ws.append(["A1",    "B",   "C",   "D1",   "E1"])
        ws.append(["A1",    "B1",   "C",   "D1",   "E2"])
        ws.append(["A2",    "B1",   "C",   "D1",   "E2"])
        ws.append(["A2",    "B1",   "C",   "D1",   "E2"])
        table_info = {'record': {'row_start':1, 'row_end' : 5},}
        sort_key_row_index = [0, 1, 2] # 根据A,B排序并合并
        table_merge = Table_Merge(ws, table_info, sort_key_row_index)
        table_merge.merge()
        print(sheet)
        
        #仅允许A，B,C 列merge， B的合并受A约束， C的合并受B约束
        self.assertEqual(len(ws.merged_cells.ranges), 6)
        for merged_cell in ws.merged_cells.ranges:
            if 'A1' in merged_cell:
                self.assertEqual(merged_cell.coord, "A1:A3")
            if 'A4' in merged_cell:
                self.assertEqual(merged_cell.coord, "A4:A5")
            if 'B1' in merged_cell:
                self.assertEqual(merged_cell.coord, "B1:B2")
            if 'B4' in merged_cell:
                self.assertEqual(merged_cell.coord, "B4:B5")
            if 'C1' in merged_cell:
                self.assertEqual(merged_cell.coord, "C1:C2")
            if 'C4' in merged_cell:
                self.assertEqual(merged_cell.coord, "C4:C5")
         
    

# 如果是直接运行这个文件，那么执行测试
if __name__ == '__main__':
    unittest.main()



from openpyxl import Workbook
from openpyxl.styles import Alignment, Border, Side, Font


class Table_Attr:
    def __init__(self, worksheet, table_info, sort_key_row_index):
        self.__worksheet = worksheet
        self.__table_info = table_info
        # 设置边框属性
        self.__border = Border(left=Side(style='thin'),
                        right=Side(style='thin'),
                        top=Side(style='thin'),
                        bottom=Side(style='thin'))
        pass

    def __set_header_attr(self, row_start, row_end, col_start, col_end):
        # 设置居中对齐属性
        alignment = Alignment(horizontal='center', vertical='center')        

        # 遍历指定行范围的所有单元格        
        for row in self.__worksheet.iter_rows(min_row=row_start, max_row=row_end, min_col=col_start, max_col=col_end):
            for cell in row:
                # 设置居中对齐属性
                cell.alignment = alignment

                # 设置边框属性
                cell.border = self.__border

                # 获取当前单元格的字体属性
                current_font = cell.font

                # 创建新的字体属性，大小比当前字体大2号，且加粗
                new_font = Font(name=current_font.name,
                                size=current_font.size + 2 if current_font.size else None,
                                bold=True)

                # 设置新的字体属性
                cell.font = new_font
                
    def __set_record_attr(self, row_start, row_end, col_start, col_end):
        
        # 设置左对齐属性
        alignment = Alignment(horizontal='left', vertical='center')
        
        # 遍历指定行范围和列范围的所有单元格
        for row in self.__worksheet.iter_rows(min_row=row_start, max_row=row_end, min_col=col_start, max_col=col_end):
            for cell in row:
                # 设置居中对齐属性
                cell.alignment = alignment

                # 设置边框属性
                cell.border = self.__border        

    
    
    def __calculate_string_width(self, s):
        # 计算字符串的宽度，其中中文字符计为3，英文字符计为1(header里面有中文字符，且header字体比record大2号)
        return sum(3 if '\u4e00' <= char <= '\u9fff' else 1 for char in s)

    def __set_col_cell_width(self, row_start, row_end, col_start, col_end):
        # 初始化一个列表来存储每一列的最大字符宽度
        max_col_widths = [0] * self.__worksheet.max_column

        # 遍历指定行范围的所有单元格
        for row in self.__worksheet.iter_rows(min_row=row_start, max_row=row_end, min_col=col_start, max_col=col_end, values_only=True):
            for col_index, cell_value in enumerate(row):
                # 如果单元格的值是数字，则计算其字符宽度
                if isinstance(cell_value, (int, float)):
                    # 使用自定义函数计算数字宽度
                    string_width = self.__calculate_string_width(str(cell_value))
                    # 更新该列的最大字符宽度
                    max_col_widths[col_index] = max(max_col_widths[col_index], string_width)    
                    
                # 如果单元格的值是字符串，则计算其字符宽度
                if isinstance(cell_value, str):
                    # 使用自定义函数计算字符串宽度
                    string_width = self.__calculate_string_width(cell_value)
                    # 更新该列的最大字符宽度
                    max_col_widths[col_index] = max(max_col_widths[col_index], string_width)

        # 遍历每一列，并根据最大字符宽度调整列宽
        for col_index, col_width in enumerate(max_col_widths):
            # 你可以根据需要调整这个乘数，以获得所需的列宽
            adjusted_width = col_width * 1.2
            
            # 获取列字母
            col_letter = chr(col_index + 65)
            
            # 获取当前列的宽度
            current_width = self.__worksheet.column_dimensions[col_letter].width
            
            # 如果调整的宽度大于原先单元格的宽度，则进行调整
            if adjusted_width > current_width or self.__table_info['first_table']:
                # 设置列宽
                self.__worksheet.column_dimensions[col_letter].width = adjusted_width



    def set_attr(self):        
        if 'header' in self.__table_info:
            header_info = self.__table_info['header']
            self.__set_header_attr(header_info['row_start'], header_info['row_end'], 1, self.__table_info['col_width'])
                
       
        if 'record' in self.__table_info:
            record_info = self.__table_info['record']
            self.__set_record_attr(record_info['row_start'], record_info['row_end'], 1, self.__table_info['col_width'])
            
        if 'header' in self.__table_info and 'record' in self.__table_info:
            self.__set_col_cell_width(header_info['row_start'], record_info['row_end'], 1, self.__table_info['col_width'])
            
        
 


from setuptools import setup, find_packages
from ptable import __version__


print(__version__)

# 读取README.md文件内容
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
    
setup(
    name='xlsx_ptable',
    version=__version__,
    packages=find_packages(),
    install_requires=[
        # 列出项目的依赖包，例如：
        'openpyxl==3.1.2',
        'PyYAML',
    ],
    author='Peng Gu',
    author_email='gu.peng@intellif.com',
    description='Extract information from log files and create Excel spreadsheets.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/gupeng1978/tools',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)



from openpyxl import Workbook


class Table_Merge:
    def __init__(self, worksheet, table_info, sort_key_row_index):
        self.__worksheet = worksheet
        self.__table_info = table_info
        self.__sort_key_row_index = sort_key_row_index
        pass
    
    #  header merge by row
    def __merge_cells_by_row(self, row_start, row_end):
        worksheet = self.__worksheet
        for row_cells in worksheet.iter_rows(min_row=row_start, max_row=row_end):
            start_col = None
            value_to_merge = None
            for col, cell in enumerate(row_cells, start=1):
                if cell.value == value_to_merge:
                    continue
                if start_col and col - 1 > start_col:
                    worksheet.merge_cells(start_row=cell.row, start_column=start_col, end_row=cell.row, end_column=col - 1)
                start_col = col
                value_to_merge = cell.value
            if start_col and col > start_col:
                worksheet.merge_cells(start_row=cell.row, start_column=start_col, end_row=cell.row, end_column=col)
    
    # header merge by col            
    def __merge_cells_by_col(self, row_start, row_end):
        worksheet = self.__worksheet
        for col_cells in worksheet.iter_cols(min_row=row_start, max_row=row_end):
            start_row = None
            value_to_merge = None
            for row, cell in enumerate(col_cells, start=row_start):
                if cell.value == value_to_merge:
                    continue
                if start_row and row - 1 > start_row:
                    worksheet.merge_cells(start_row=start_row, start_column=cell.column, end_row=row - 1, end_column=cell.column)
                start_row = row
                value_to_merge = cell.value
            if start_row and row > start_row:
                worksheet.merge_cells(start_row=start_row, start_column=cell.column, end_row=row, end_column=cell.column)
    
    
    def __merge_cells_for_records(self, excel_row_start, excel_row_end):
        worksheet = self.__worksheet
        sort_key_row_index = self.__sort_key_row_index
        # 遍历每一列
        allowed_seg_table = [{'start':excel_row_start, 'end':excel_row_end}]
        next_allowed_seg_table = []
        
        def get_segments_from_list(req_seg):
            allowed_segments = []
            
            # 遍历允许的分段表
            for allowed_seg in allowed_seg_table:
                # 检查请求分段与允许分段的交集
                start = max(req_seg['start'], allowed_seg['start'])
                end = min(req_seg['end'], allowed_seg['end'])
                
                # 如果存在交集，则添加到允许的分段列表中
                if start < end:
                    allowed_segments.append({'start': start, 'end': end})
            
            return allowed_segments
        
        def try_merged(start_row, end_row, column) :
            allowed_seg = get_segments_from_list({'start':start_row, 'end':end_row})
            if allowed_seg :
                for seg in allowed_seg:
                    worksheet.merge_cells(
                        start_row=seg['start'], start_column=column,
                        end_row=seg['end'], end_column=column
                    )
                next_allowed_seg_table.append(seg)        
            pass
        
        for col in worksheet.iter_cols(min_row=excel_row_start, max_row=excel_row_end):
            
            # 只合并sort_key_row_index所在的列
            if col[0].column - 1 not in sort_key_row_index:
                continue
            next_allowed_seg_table = []
            
            col_number = col[0].column
            combine_start = excel_row_start
            previous_value = None
            # 遍历列中的每一行
            for row_index, cell in enumerate(col, start=excel_row_start):
                if previous_value is None:
                    previous_value = cell.value
                    continue
                
                # 如果当前值与前一个值相同，继续遍历
                if cell.value == previous_value:
                    continue
                
                # 如果当前值与前一个值不同，检查是否需要合并
                if row_index - combine_start > 1:
                    try_merged(combine_start, row_index - 1, cell.column)
                
                # 更新合并的起始行和前一个值
                combine_start = row_index
                previous_value = cell.value
            
            # 检查最后一组单元格是否需要合并
            if excel_row_end - combine_start > 0:
                try_merged(combine_start, excel_row_end, cell.column)
            
            # 更新下一次迭代允许的分段    
            allowed_seg_table = next_allowed_seg_table
    
    
    def merge(self):        
        if 'header' in self.__table_info:
            header_info = self.__table_info['header']
            self.__merge_cells_by_row(header_info['row_start'], header_info['row_end'])
            self.__merge_cells_by_col(header_info['row_start'], header_info['row_end'])
                
       
        if 'record' in self.__table_info:
            record_info = self.__table_info['record']
            self.__merge_cells_for_records(record_info['row_start'], record_info['row_end'])
        
 